@@ -3,6 +3,7 @@
 Load the files detected from the scanner
 """
 # Import Python libs
+import asyncio
 import os
 import sys
 import inspect
@@ -10,13 +11,15 @@ import importlib
 import importlib.util
 import importlib.machinery
 import traceback as stdlib_traceback
+import types
+from typing import Any, Dict, List, Tuple
 
 # Import pop libs
 import pop.exc
 import pop.contract
 
 
-class LoadError:
+class LoadError(Exception):
     """
     Errors from the loader are contained herein
     """
@@ -58,17 +61,22 @@ class LoadError:
         return "<{} edict={!r}>".format(self.__class__.__name__, self.edict)
 
 
-def load_mod(modname, form, path):
+def load_mod(modname: str, form: str, path: str) -> "LoadedMod":
     """
     Load a single module
+    :param form: The name of the loader module
+    :param modname: The name of the module to get from the loader
+    :param path: The package to use as the anchor point from which to resolve the
+        relative import to an absolute import.
     """
     this = sys.modules[__name__]
     return getattr(this, form)(modname, path)
 
 
-def _generate_module(name):
+def _generate_module(name: str) -> types.ModuleType:
     """
     Generate a module at runtime and insert it in sys.modules
+    :param name: The name of the module to create
     """
     if name in sys.modules:
         return sys.modules[name]
@@ -76,13 +84,14 @@ def _generate_module(name):
     code = "'''POP sub auto generated parent module for {0}'''".format(
         name.split(".")[-1]
     )
-    module = stdlib_imp.new_module(name)
+    # Create a new module that is not entered into sys.modules
+    module = types.ModuleType(name)
     exec(code, module.__dict__)  # pylint: disable=exec-used
     sys.modules[name] = module
     return module
 
 
-def _populate_sys_modules(mod):
+def _populate_sys_modules(mod: "LoadedMod"):
     """
     This is a hack to populate sys.modules with the modules that pop loads
     while making sure that parent modules have the attribute for the child
@@ -101,9 +110,12 @@ def _populate_sys_modules(mod):
         gen_mod = gen_child_mod
 
 
-def ext(modname, path):
+def ext(modname: str, path: str) -> "LoadedMod" or LoadError:
     """
     Attempt to load the named python modules
+    :param modname: The name of the module to get from the loader
+    :param path: The package to use as the anchor point from which to resolve the
+        relative import to an absolute import.
     """
     modname = ".".join(modname.split(".")[:-1])
     if modname in sys.modules:
@@ -121,9 +133,12 @@ def ext(modname, path):
         )
 
 
-def python(modname, path):
+def python(modname: str, path: str) -> "LoadedMod" or LoadError:
     """
     Attempt to load the named python modules
+    :param modname: The name of the module to get from the loader
+    :param path: The package to use as the anchor point from which to resolve the
+        relative import to an absolute import.
     """
     if modname in sys.modules:
         return sys.modules[modname]
@@ -138,18 +153,36 @@ def python(modname, path):
         )
 
 
-def load_virtual(hub, virtual, mod, bname):
+def _base_name(bname: str, mod: "LoadedMod") -> Tuple[str, str]:
     """
-    Run the virtual function to name the module and check for all loader
-    errors
+    Find the basename and alias of a loader module
+    :param bname: The base name of the mod's path
+    :param mod: A loader module or a LoadError if the module didn't load
     """
     base_name = os.path.basename(bname)
     if "." in base_name:
         base_name = base_name.split(".")[0]
-    try:
-        name = mod.__virtualname__
-    except Exception:  # pylint: disable=broad-except
-        name = base_name
+    return base_name, getattr(mod, "__virtualname__", base_name)
+
+
+def _load_virtual(
+    hub: "pop.hub.Hub",
+    virtual: bool,
+    mod: "LoadedMod" or LoadError,
+    bname: str,
+    vtype: str,
+) -> Dict[str, Any]:
+    """
+    Run the virtual function to name the module and check for all loader
+    errors
+    :param hub: The redistributed pop central hub
+    :param virtual: Toggle whether or not to process __virtual__ functions
+    :param mod: A loader module or a LoadError if the module didn't load
+    :param bname: The base name of the mod's path
+    :param vtype: The name of the virtual function to call on the module I.E. __virtual__ or __sub_virtual__
+    """
+    base_name, name = _base_name(bname, mod)
+
     if isinstance(mod, LoadError):
         # The mod is a LoadError instance.
         # Return the load error with name as the base_name because another
@@ -161,14 +194,17 @@ def load_virtual(hub, virtual, mod, bname):
         # __virtual__ is not to be processed. Return now!
         return {"name": base_name}
 
-    if not hasattr(mod, "__virtual__"):
+    if not hasattr(mod, vtype):
         # No __virtual__ processing is required.
         # Return the mod's name as the defined __virtualname__ if defined,
         # else, the base_name
         return {"name": name}
 
     try:
-        vret = mod.__virtual__(hub)
+        vret = getattr(mod, vtype)(hub)
+        # If the __virtual__ function was asynchronous then run it in an asyncio loop
+        if asyncio.iscoroutine(vret):
+            vret = asyncio.get_running_loop().run_until_complete(vret)
     except Exception as exc:  # pylint: disable=broad-except
         err = LoadError(
             "Virtual threw exception in mod {}".format(bname),
@@ -180,6 +216,12 @@ def load_virtual(hub, virtual, mod, bname):
         # but also return the vname information
         return {"name": base_name, "vname": name, "error": err}
 
+    verror = vret
+    if isinstance(vret, tuple):
+        if len(vret) > 1:
+            verror = vret[1]
+        vret = vret[0]
+
     if vret is True:
         # No problems occurred, module is allowed to load
         # Return the mod's name as the defined __virtualname__ if defined,
@@ -188,38 +230,76 @@ def load_virtual(hub, virtual, mod, bname):
 
     if vret is False:
         # __virtual__ explicitly disabled the loading of this module
-        err = LoadError("Module {} returned virtual FALSE".format(bname), verror=vret)
+        err = LoadError("Module {} returned virtual FALSE".format(bname), verror=verror)
         # Return the load error with name as the base_name because another
         # module is still allowed to load under the same __virtualname__
         # but also return the vname information
         return {"name": base_name, "vname": name, "error": err}
 
     # Anything else besides True/False should be considered a LoadError
-    err = LoadError("Module {} returned virtual error".format(bname), verror=vret)
+    err = LoadError("Module {} returned virtual error".format(bname), verror=verror)
     # Return the load error with name as the base_name because another
     # module is still allowed to load under the same __virtualname__
     # but also return the vname information
     return {"name": base_name, "vname": name, "error": err}
 
 
-def mod_init(sub, mod, mod_name):
+def load_virtual(
+    hub: "pop.hub.Hub", virtual: bool, mod: "LoadedMod" or LoadError, bname: str
+) -> Dict[str, Any]:
+    """
+    Run the __virtual__ function to name the module and check for all loader errors
+    :param hub: The redistributed pop central hub
+    :param virtual: Toggle whether or not to process __virtual__ functions
+    :param mod: A loader module or a LoadError if the module didn't load
+    :param bname: The base name of the mod's path
+    """
+    return _load_virtual(hub, virtual, mod, bname, "__virtual__")
+
+
+def load_sub_virtual(
+    hub: "pop.hub.Hub", virtual: bool, mod: "LoadedMod" or LoadError, bname: str
+) -> Dict[str, Any]:
+    """
+    Run the __sub_virtual__ function to name the module and check for all loader errors
+    :param hub: The redistributed pop central hub
+    :param virtual: Toggle whether or not to process __virtual__ functions
+    :param mod: A loader module or a LoadError if the module didn't load
+    :param bname: The base name of the mod's path
+    """
+    _, name = _base_name(bname, mod)
+    if name != "init":
+        return {"name": name}
+    return _load_virtual(hub, virtual, mod, bname, "__sub_virtual__")
+
+
+def mod_init(sub: "pop.hub.Sub", mod: "LoadedMod", mod_name: str):
     """
     Process module's __init__ function if defined
+    :param sub: The pop object that contains the loaded module data
+    :param mod: A loader modul
+    :param mod_name: The name of the module to get from the loader
     """
     if "__init__" in dir(mod):
         init = pop.contract.Contracted(
             sub._hub,
-            contracts=None,
+            contracts=[],
             func=mod.__init__,
             ref=f"{sub._subname}.{mod_name}",
             name="__init__",
         )
-        init()
+        ret = init()
+        # If the __init__ function was asynchronous then run it in an asyncio loop
+        if asyncio.iscoroutine(ret):
+            asyncio.get_running_loop().run_until_complete(ret)
 
 
-def sub_alias(this_sub, mod, mod_name):
+def sub_alias(this_sub: "pop.hub.Sub", mod: "LoadedMod", mod_name: str):
     """
     Check the sub alias settings and apply the alias names locally so they can be gathered into the higher level object on the hub
+    :param this_sub: The pop object that contains the loaded module data
+    :param mod: A loader module
+    :param mod_name: The name of the module to get from the loader
     """
     if mod_name == "init":
         alias = getattr(mod, "__sub_alias__", [])
@@ -227,10 +307,19 @@ def sub_alias(this_sub, mod, mod_name):
             this_sub._alias = alias
 
 
-def prep_loaded_mod(this_sub, mod, mod_name, contracts):
+def prep_loaded_mod(
+    this_sub: "pop.hub.Sub",
+    mod: "LoadedMod",
+    mod_name: str,
+    contracts: List[pop.contract.Wrapper],
+) -> "LoadedMod":
     """
     Read the attributes of a python module and create a LoadedMod, which resolves
     aliases and omits objects that should not be exposed.
+    :param this_sub: The pop object that contains the loaded module data
+    :param mod: A loader module
+    :param mod_name: The name of the module to get from the loader
+    :param contracts: Contracts functions to add to the sub
     """
     lmod = this_sub._loaded.get(mod_name, LoadedMod(mod_name))
     ref = f"{this_sub._subname}.{mod_name}"  # getattr(hub, ref) should resolve to this module
@@ -278,21 +367,21 @@ def prep_loaded_mod(this_sub, mod, mod_name, contracts):
     return lmod
 
 
-class LoadedMod:
+class LoadedMod(types.ModuleType):
     """
     The LoadedMod class allows for the module loaded onto the sub to return
     custom sequencing, for instance it can be iterated over to return all
     functions
     """
 
-    def __init__(self, name):
-        self.__name__ = name
+    def __init__(self, name: str):
+        super().__init__(name)
         self._vars = {}
         self._funcs = {}
         self._classes = {}
         self._attrs = {}
 
-    def __getattr__(self, item):
+    def __getattr__(self, item: str):
         if item in self._attrs:
             return self._attrs[item]
         raise AttributeError(item)
