@@ -5,6 +5,7 @@ Define the JSON loader interface
 
 # Import python libs
 import json
+import pop.hub
 
 __virtualname__ = "json"
 __contracts__ = [__virtualname__]
@@ -14,7 +15,7 @@ def __virtual__(hub):
     return True
 
 
-def load(hub, path):
+def load(hub: "pop.hub.Hub", path):
     """
     Use json to read in a file
     """
@@ -26,7 +27,7 @@ def load(hub, path):
         return {}
 
 
-def render(hub, val):
+def render(hub: "pop.hub.Hub", val):
     """
     Take the string and render it in json
     """
