@@ -6,6 +6,7 @@ this translates to the registry.
 """
 # Import python libs
 import os
+import pop.hub
 
 __virtualname__ = "os"
 
@@ -18,7 +19,7 @@ def __virtual__(hub):
     return True
 
 
-def gather(hub, defaults):
+def gather(hub: "pop.hub.Hub", defaults):
     """
     Iterate over the default config data and look for os: True/str options. When set
     gather the option from environment variables is present
