@@ -3,6 +3,7 @@ Support embedding version number lookup into cli
 """
 # IMport python libs
 import importlib
+import pop.hub
 import sys
 
 
@@ -15,7 +16,7 @@ CONFIG = {
 }
 
 
-def run(hub, primary):
+def run(hub: "pop.hub.Hub", primary):
     """
     Check the version number and then exit
     """
