@@ -1,9 +1,11 @@
 """
 Used to resolve resolutions to paths on the hub
 """
+import pop.hub
+from typing import List
 
 
-def last(hub, ref):
+def last(hub: "pop.hub.Hub", ref: str) -> "pop.hub.Sub":
     """
     Takes a string that references the desired ref and returns the last object
     called out in that ref
@@ -11,7 +13,7 @@ def last(hub, ref):
     return hub.pop.ref.path(ref)[-1]
 
 
-def path(hub, ref):
+def path(hub: "pop.hub.Hub", ref: str) -> List["pop.hub.Sub"]:
     """
     Retuns a list of references up to the named ref
     """
@@ -23,12 +25,13 @@ def path(hub, ref):
     return ret
 
 
-def create(hub, ref, obj):
+def create(hub: "pop.hub.Hub", ref: str, obj: object):
     """
     Create an attribute at a given target using just a ref string and the
     object to be saved at said location. The desired location must already
     exist!
 
+    :param hub: The redistributed pop central hub
     :param ref: The dot delimited string referencing the target location to
         create the given object on the hub
     :param obj: The object to store at the given reference point
