@@ -3,9 +3,10 @@ Routines to verify the working environment etc.
 """
 # Import python libs
 import os
+import pop.hub
 
 
-def env(hub):
+def env(hub: "pop.hub.Hub"):
     """
     Verify that the directories specified in the system exist
     """
