@@ -12,9 +12,10 @@ import subprocess
 
 # Import third party libs
 import msgpack
+import pop.hub
 
 
-def __init__(hub):
+def __init__(hub: "pop.hub.Hub"):
     """
     Create constants used by the client and server side of procs
     """
@@ -26,7 +27,7 @@ def __init__(hub):
     hub.proc.WorkersTrack = {}
 
 
-def _get_cmd(hub, ind, ref, ret_ref, sock_dir):
+def _get_cmd(hub: "pop.hub.Hub", ind, ref, ret_ref, sock_dir):
     """
     Return the shell command to execute that will start up the worker
     """
@@ -39,7 +40,7 @@ def _get_cmd(hub, ind, ref, ret_ref, sock_dir):
     return cmd
 
 
-def mk_proc(hub, ind, workers, ret_ref, sock_dir):
+def mk_proc(hub: "pop.hub.Hub", ind, workers, ret_ref, sock_dir):
     """
     Create the process and add it to the passed in workers dict at the
     specified index
@@ -52,7 +53,9 @@ def mk_proc(hub, ind, workers, ret_ref, sock_dir):
     workers[ind]["pid"] = workers[ind]["proc"].pid
 
 
-async def pool(hub, num, name="Workers", callback=None, sock_dir=None):
+async def pool(
+    hub: "pop.hub.Hub", num, name: str = "Workers", callback=None, sock_dir=None
+):
     """
     Create a new local pool of process based workers
 
@@ -89,7 +92,7 @@ async def pool(hub, num, name="Workers", callback=None, sock_dir=None):
     # asyncio.ensure_future(hub.proc.init.maintain(name))
 
 
-async def maintain(hub, name):
+async def maintain(hub: "pop.hub.Hub", name):
     """
     Keep an eye on these processes
     """
@@ -101,7 +104,7 @@ async def maintain(hub, name):
         await asyncio.sleep(2)
 
 
-def mk_tracker(hub):
+def mk_tracker(hub: "pop.hub.Hub"):
     """
     Create the process tracker, this simply makes a data structure to hold
     process references and sets them to be terminated when the system is
@@ -111,7 +114,7 @@ def mk_tracker(hub):
     atexit.register(hub.proc.init.clean)
 
 
-def clean(hub):
+def clean(hub: "pop.hub.Hub"):
     """
     Clean up the processes registered in the tracker
     """
@@ -120,7 +123,7 @@ def clean(hub):
             workers[ind]["proc"].terminate()
 
 
-def ret_work(hub, callback):
+def ret_work(hub: "pop.hub.Hub", callback):
     async def work(reader, writer):
         """
         Process the incoming work
